@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2010 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -32,24 +32,25 @@
 */
 
 /* API and ABI is unstable */
-#ifndef _uim_h_included_
-#define _uim_h_included_
+#ifndef UIM_H
+#define UIM_H
 
-#include <stdio.h>
+#include <stdlib.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 #define UIM_VERSION_MAJOR      (1)
-#define UIM_VERSION_MINOR      (4)
-#define UIM_VERSION_PATCHLEVEL (1)
+#define UIM_VERSION_MINOR      (6)
+#define UIM_VERSION_PATCHLEVEL (0)
 #define UIM_VERSION_REQUIRE(major, minor, patchlevel)			     \
   ((major) < UIM_VERSION_MAJOR						     \
    || ((major) == UIM_VERSION_MAJOR && (minor) < UIM_VERSION_MINOR)	     \
    || ((major) == UIM_VERSION_MAJOR && (minor) == UIM_VERSION_MINOR	     \
        && (patchlevel) <= UIM_VERSION_PATCHLEVEL))
 
+#ifndef UIM_BOOL_DEFINED
 /*
  * A boolean type for uim to explicitly indicate intention about values
  *
@@ -64,12 +65,14 @@ typedef int uim_bool;
 #define UIM_FALSE 0
 #define UIM_TRUE 1
 
+#define UIM_BOOL_DEFINED 1
+#endif /* UIM_BOOL_DEFINED */
+
 
 typedef struct uim_context_ *uim_context;
 
 typedef struct uim_candidate_ *uim_candidate;
 
-/* FIXME: We need reorganize of key event handling...*/
 enum UKey {
   UKey_0 = 48,
   UKey_1 = 49,
@@ -82,10 +85,12 @@ enum UKey {
   UKey_8 = 56,
   UKey_9 = 57,
   UKey_Yen = 165,
+
   UKey_Escape = 256,
   UKey_Tab,
   UKey_Backspace,
   UKey_Delete,
+  UKey_Insert,
   UKey_Return,
   UKey_Left,
   UKey_Up ,
@@ -95,8 +100,15 @@ enum UKey {
   UKey_Next , /* page down */
   UKey_Home,
   UKey_End,
+
   UKey_Multi_key, /* multi-key character compose */
+  UKey_Codeinput,
+  UKey_SingleCandidate,
+  UKey_MultipleCandidate,
+  UKey_PreviousCandidate,
   UKey_Mode_switch, /* charcter set switch */
+
+  /* Japanese keyboard */
   UKey_Kanji, /* kanji, kanji convert */
   UKey_Muhenkan, /* cancel conversion */
   UKey_Henkan_Mode, /* start/stop conversion */
@@ -114,6 +126,25 @@ enum UKey {
   UKey_Kana_Shift,
   UKey_Eisu_Shift, /* alphanumeric shift */
   UKey_Eisu_toggle, /* alphanumeric toggle */
+
+  /* Korean keyboard */
+  UKey_Hangul,
+  UKey_Hangul_Start,
+  UKey_Hangul_End,
+  UKey_Hangul_Hanja,
+  UKey_Hangul_Jamo,
+  UKey_Hangul_Romaja,
+  UKey_Hangul_Codeinput,
+  UKey_Hangul_Jeonja,
+  UKey_Hangul_Banja,
+  UKey_Hangul_PreHanja,
+  UKey_Hangul_PostHanja,
+  UKey_Hangul_SingleCandidate,
+  UKey_Hangul_MultipleCandidate,
+  UKey_Hangul_PreviousCandidate,
+  UKey_Hangul_Special,
+
+  /* function keys */
   UKey_F1,
   UKey_F2,
   UKey_F3,
@@ -149,70 +180,6 @@ enum UKey {
   UKey_F33,
   UKey_F34,
   UKey_F35, /* X, Gtk and Qt supports up to F35 */
-  /* non-standard platform specific keys (e.g. Zaurus PDA) */
-  UKey_Private1,
-  UKey_Private2,
-  UKey_Private3,
-  UKey_Private4,
-  UKey_Private5,
-  UKey_Private6,
-  UKey_Private7,
-  UKey_Private8,
-  UKey_Private9,
-  UKey_Private10,
-  UKey_Private11,
-  UKey_Private12,
-  UKey_Private13,
-  UKey_Private14,
-  UKey_Private15,
-  UKey_Private16,
-  UKey_Private17,
-  UKey_Private18,
-  UKey_Private19,
-  UKey_Private20,
-  UKey_Private21,
-  UKey_Private22,
-  UKey_Private23,
-  UKey_Private24,
-  UKey_Private25,
-  UKey_Private26,
-  UKey_Private27,
-  UKey_Private28,
-  UKey_Private29,
-  UKey_Private30,
-  /* this part is especially unstable */
-  /* modifier keys: "_key" suffix should be removed in the future */
-  UKey_Shift_key,
-  UKey_Control_key,
-  UKey_Alt_key,
-  UKey_Meta_key,
-  UKey_Super_key,
-  UKey_Hyper_key,
-  UKey_Insert,
-
-  /* These new keys added at uim 1.3.0 are placed here to keep (virtually)
-   * backward-compatible ABI. It will be relocated to appropriate place when
-   * an backward-incompatible ABI change has occur. */
-  UKey_Codeinput,
-  UKey_SingleCandidate,
-  UKey_MultipleCandidate,
-  UKey_PreviousCandidate,
-
-  UKey_Hangul,
-  UKey_Hangul_Start,
-  UKey_Hangul_End,
-  UKey_Hangul_Hanja,
-  UKey_Hangul_Jamo,
-  UKey_Hangul_Romaja,
-  UKey_Hangul_Codeinput,
-  UKey_Hangul_Jeonja,
-  UKey_Hangul_Banja,
-  UKey_Hangul_PreHanja,
-  UKey_Hangul_PostHanja,
-  UKey_Hangul_SingleCandidate,
-  UKey_Hangul_MultipleCandidate,
-  UKey_Hangul_PreviousCandidate,
-  UKey_Hangul_Special,
 
   /* dead keys */
   UKey_Dead_Grave,
@@ -235,7 +202,7 @@ enum UKey {
   UKey_Dead_Hook,
   UKey_Dead_Horn,
 
-  /* Kana keys */
+  /* Japanese Kana keys */
   UKey_Kana_Fullstop,
   UKey_Kana_OpeningBracket,
   UKey_Kana_ClosingBracket,
@@ -300,11 +267,62 @@ enum UKey {
   UKey_Kana_VoicedSound,
   UKey_Kana_SemivoicedSound,
 
-  UKey_Caps_Lock,
+  /* non-standard platform specific keys (e.g. Zaurus PDA) */
+  UKey_Private1,
+  UKey_Private2,
+  UKey_Private3,
+  UKey_Private4,
+  UKey_Private5,
+  UKey_Private6,
+  UKey_Private7,
+  UKey_Private8,
+  UKey_Private9,
+  UKey_Private10,
+  UKey_Private11,
+  UKey_Private12,
+  UKey_Private13,
+  UKey_Private14,
+  UKey_Private15,
+  UKey_Private16,
+  UKey_Private17,
+  UKey_Private18,
+  UKey_Private19,
+  UKey_Private20,
+  UKey_Private21,
+  UKey_Private22,
+  UKey_Private23,
+  UKey_Private24,
+  UKey_Private25,
+  UKey_Private26,
+  UKey_Private27,
+  UKey_Private28,
+  UKey_Private29,
+  UKey_Private30,
+
+  /* modifier keys */
+  UKey_Shift = 0x8000,
+  UKey_Control,
+  UKey_Alt,
+  UKey_Meta,
+  UKey_Super,
+  UKey_Hyper,
+
+  /* lock modifier keys: unstable */
+  UKey_Caps_Lock = 0x9000,
   UKey_Num_Lock,
   UKey_Scroll_Lock,
 
-  UKey_Other = 1000
+#if 1
+  /* Deprecated. Please replace with UKey_Shift and so on. */
+  UKey_Shift_key = UKey_Shift,
+  UKey_Control_key = UKey_Control,
+  UKey_Alt_key = UKey_Alt,
+  UKey_Meta_key = UKey_Meta,
+  UKey_Super_key = UKey_Super,
+  UKey_Hyper_key = UKey_Hyper,
+#endif
+
+  UKey_Other = 0x10000
 };
   
 enum UKeyModifier {
@@ -560,6 +578,15 @@ uim_press_key(uim_context uc, int key, int state);
 int
 uim_release_key(uim_context uc, int key, int state);
 
+/**
+ * Change client encoding of an input context.
+ *
+ * @param uc input context
+ * @param encoding client encoding name
+ *
+ * @see uim_create_context
+ */
+void uim_set_client_encoding(uim_context uc, const char *encoding);
 
 
 /* im list */
@@ -580,7 +607,7 @@ int uim_get_nr_im(uim_context uc);
  * @param uc input context
  * @param nth index of input method.
  *
- * @return name of nth input method
+ * @return name of nth input method. only valid until next uim API call.
  */
 const char *uim_get_im_name(uim_context uc, int nth);
 
@@ -590,7 +617,7 @@ const char *uim_get_im_name(uim_context uc, int nth);
  * @warning you must not free the result.
  *
  * @param uc input context
- * @param nth index of input method
+ * @param nth index of input method. only valid until next uim API call.
  */
 const char *uim_get_im_language(uim_context uc, int nth);
 
@@ -600,7 +627,7 @@ const char *uim_get_im_language(uim_context uc, int nth);
  * @warning you must not free the result.
  *
  * @param uc input context
- * @param nth index of input method
+ * @param nth index of input method. only valid until next uim API call.
  */
 const char *uim_get_im_short_desc(uim_context uc, int nth);
 
@@ -612,7 +639,7 @@ const char *uim_get_im_short_desc(uim_context uc, int nth);
  * @param uc input context
  * @param nth index of input method
  *
- * @return nth input method's encoding
+ * @return nth input method's encoding. only valid until next uim API call.
  */
 const char *uim_get_im_encoding(uim_context uc, int nth);
 
@@ -624,7 +651,7 @@ const char *uim_get_im_encoding(uim_context uc, int nth);
  *
  * @param localename locale name
  *
- * @return input method name.
+ * @return input method name. only valid until next uim API call.
  */
 const char *uim_get_default_im_name(const char *localename);
 
@@ -635,7 +662,7 @@ const char *uim_get_default_im_name(const char *localename);
  *
  * @param localename locale name
  *
- * @return input method name.
+ * @return input method name. only valid until next uim API call.
  */
 const char *uim_get_im_name_for_locale(const char *localename);
 
@@ -652,18 +679,22 @@ const char *uim_get_im_name_for_locale(const char *localename);
  *
  * @see uim_create_context
  */
-int uim_set_candidate_selector_cb(uim_context uc,
-				  void (*activate_cb)(void *ptr, int nr, int display_limit),
-				  void (*select_cb)(void *ptr, int index),
-				  void (*shift_page_cb)(void *ptr, int direction),
-				  void (*deactivate_cb)(void *ptr));
+void uim_set_candidate_selector_cb(uim_context uc,
+                                   void (*activate_cb)(void *ptr,
+                                                       int nr,
+                                                       int display_limit),
+                                   void (*select_cb)(void *ptr, int index),
+                                   void (*shift_page_cb)(void *ptr,
+                                                         int direction),
+                                   void (*deactivate_cb)(void *ptr));
 
 /**
  * Get candidate data.
  *
  * @param uc input context
  * @param index index of the candidate you want to get
- * @param accel_enumeration_hint
+ * @param accel_enumeration_hint index of the first candidate displayed in
+ * the candidate selector
  *
  * @warning You must free the result by uim_candidate_free
  *
@@ -778,7 +809,7 @@ uim_set_mode_cb(uim_context uc, void (*update_cb)(void *ptr,
 /* mode list */
 int
 uim_get_nr_modes(uim_context uc);
-/* you must not free the result */
+/* you must not free the result. and only valid until next uim API call */
 const char *
 uim_get_mode_name(uim_context uc, int nth);
 void
@@ -796,7 +827,7 @@ uim_set_mode_list_update_cb(uim_context uc,
  * completely, adding separated surrounding text deletion interface is better
  * solution at now.  -- YamaKen 2006-10-07
  *
- * http://anonsvn.freedesktop.org/svn/uim/branches/composer/scm/event.scm
+ * http://uim.googlecode.com/svn/branches/composer/scm/event.scm
  *
  * (define-event 'commit
  *   upward-event-rec-spec
@@ -888,54 +919,18 @@ void
 uim_set_configuration_changed_cb(uim_context uc,
 				 void (*changed_cb)(void *ptr));
 
-/*
- * Set callback functions called when IM-switching of specific set of context
- * is requested.
- *
- * When the functions are called back, bridges should re-initialize the
- * specified input contexts with the IM specified by 2nd argument
- * 'name'. Since the re-initialization method of specified contexts vary for
- * each IM environment, it is delegated to bridges via the callback. For
- * example, ordinary desktop system should send the helper message
- * im_change_whole_desktop in response to @a sw_system_im_cb. But in embedded
- * systems such as Qtopia, nothing to do with @a sw_system_im_cb since only
- * one input context is running on the window system and so system-global.
- *
- * @param uc input context
 
- * @param sw_app_im_cb called when re-initialization of all contexts within
- *        the application that @a uc belongs to, with specified IM is
- *        requested. 1st argument "ptr" corresponds to the 1st argument of
- *        uim_create_context, and 2nd "name" is requested idname of IM. The
- *        originating context is supposed to already be switched, and must not
- *        switched by the callback.
- * @param sw_system_im_cb called when re-initialization of all contexts
- *        running on the system that @a uc is running on, with specified IM is
- *        requested. The originating context is supposed to already be
- *        switched.
- */
-void
-uim_set_im_switch_request_cb(uim_context uc,
-			     void (*sw_app_im_cb)(void *ptr, const char *name),
-			     void (*sw_system_im_cb)(void *ptr, const char *name));
+/* For plugins implementation. Bridges should not use these functions. */
+void uim_fatal_error(const char *msg);  /* Disables uim */
+void *uim_malloc(size_t size);
+void *uim_realloc(void *p, size_t size);
+void *uim_calloc(size_t nmemb, size_t size);
+char *uim_strdup(const char *s);
+int uim_asprintf(char **, const char *, ...);
 
-/* Utility functions */
-int
-uim_ipc_open_command(int old_pid, FILE **read_handler, FILE **write_handler, const char *command);
-int
-uim_ipc_open_command_with_option(int old_pid, FILE **read_handler, FILE **write_handler, const char *command, const char *option);
-char *
-uim_ipc_send_command(int *pid, FILE **read_handler,
-		     FILE **write_handler, const char *command, const char *str);
-
-/* will be deprecated. use custom API (uim-custom.h) instead */
-char *uim_symbol_value_str(const char *symbol_str);
-
-/* an uim_code_converter implementation using iconv */
-extern struct uim_code_converter *uim_iconv;
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* _uim_h_included_ */
+#endif /* UIM_H */
