@@ -1,5 +1,5 @@
 ;; 
-;;  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+;;  Copyright (c) 2005-2010 uim Project http://code.google.com/p/uim/
 ;;
 ;;  All rights reserved.
 ;;
@@ -618,7 +618,8 @@
 (defun uim-window-changed ()
   (let (candidate-displayed)
   
-    (when (and uim-candidate-displayed
+    (when (and (not uim-candidate-in-echo-region)
+	       uim-candidate-displayed
 	       uim-candidate-display-inline)
       (setq candidate-displayed uim-candidate-displayed)
 
