@@ -1,7 +1,7 @@
 ;;; anthy.scm: Anthy for uim.
 ;;; charset: EUC-JP
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2010 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -35,8 +35,6 @@
 (require "util.scm")
 (require "ustr.scm")
 (require "japanese.scm")
-(require "japanese-kana.scm")
-(require "japanese-azik.scm")
 (require-custom "generic-key-custom.scm")
 (require-custom "anthy-custom.scm")
 (require-custom "anthy-key-custom.scm")
@@ -57,6 +55,8 @@
 (define anthy-input-rule-roma 0)
 (define anthy-input-rule-kana 1)
 (define anthy-input-rule-azik 2)
+(define anthy-input-rule-act 3)
+(define anthy-input-rule-kzik 4)
 
 (define anthy-candidate-type-katakana -2)
 (define anthy-candidate-type-hiragana -3)
@@ -256,6 +256,7 @@
 		      anthy-input-rule-kana))
 		 (lambda (ac)
 		   (anthy-prepare-input-rule-activation ac)
+                   (require "japanese-kana.scm")
 		   (anthy-context-set-input-rule! ac anthy-input-rule-kana)
 		   (anthy-context-change-kana-mode! ac (anthy-context-kana-mode ac))
 		   (anthy-context-set-alnum! ac #f)
@@ -278,11 +279,49 @@
 		      anthy-input-rule-azik))
 		 (lambda (ac)
 		   (anthy-prepare-input-rule-activation ac)
+                   (require "japanese-azik.scm")
 		   (rk-context-set-rule! (anthy-context-rkc ac)
 					 ja-azik-rule)
 		   (japanese-roma-set-yen-representation)
 		   (anthy-context-set-input-rule! ac anthy-input-rule-azik)))
 
+(register-action 'action_anthy_kzik
+;;		 (indication-alist-indicator 'action_anthy_kzik
+;;					     anthy-kana-input-method-indication-alist)
+		 (lambda (ac)
+		   '(ja_kzik
+		     ""
+		     "KZIK"
+		     "KZIKĥ޻ϥ⡼"))
+		 (lambda (ac)
+		   (= (anthy-context-input-rule ac)
+		      anthy-input-rule-kzik))
+		 (lambda (ac)
+		   (anthy-prepare-input-rule-activation ac)
+                   (require "japanese-kzik.scm")
+		   (rk-context-set-rule! (anthy-context-rkc ac)
+					 ja-kzik-rule)
+		   (japanese-roma-set-yen-representation)
+		   (anthy-context-set-input-rule! ac anthy-input-rule-kzik)))
+
+(register-action 'action_anthy_act
+		 (lambda (ac)
+		   '(ja_act
+		     ""
+		     "ACT"
+		     "ACTĥ޻ϥ⡼"))
+		 (lambda (ac)
+		   (= (anthy-context-input-rule ac)
+		      anthy-input-rule-act))
+		 (lambda (ac)
+		   (anthy-prepare-input-rule-activation ac)
+                   (require "japanese-act.scm")
+		   (rk-context-set-rule! (anthy-context-rkc ac)
+					 ja-act-rule)
+		   (japanese-roma-set-yen-representation)
+		   (anthy-context-set-input-rule! ac anthy-input-rule-act)))
+
+
 ;; Update widget definitions based on action configurations. The
 ;; procedure is needed for on-the-fly reconfiguration involving the
 ;; custom API
@@ -339,15 +378,6 @@
      (anthy-context-set-preconv-ustr! ac (ustr-new '()))
      (anthy-context-set-raw-ustr! ac (ustr-new '()))
      (anthy-context-set-segments! ac (ustr-new '()))
-
-     ;; 2004-08-26 Takuro Ashie <ashie@homa.ne.jp>
-     ;;   * I think load-kana-table should be marked as depracated.
-     ;;     Because it is a little violent (it overwrites ja-rk-rule table).
-     ;;     We should prepare a custom entry like "uim-default-input-rule"
-     ;;     instead of using-kana-table.
-     (if using-kana-table?
-	 (anthy-context-set-input-rule! ac anthy-input-rule-kana)
-	 (anthy-context-set-input-rule! ac anthy-input-rule-roma))
      ac)))
 
 (define anthy-commit-raw
@@ -400,8 +430,10 @@
 	      (string-append-map-ustr-former extract-kana preconv-str)
 	      (if convert-pending-into-kana?
 		  (if residual-kana
-		      (extract-kana residual-kana)
-		      pending)
+                    (if (list? (car residual-kana))
+                      (string-append-map extract-kana residual-kana)
+		      (extract-kana residual-kana))
+                    pending)
 		  pending)
 	      (string-append-map-ustr-latter extract-kana preconv-str))))
 	   kana)
@@ -409,8 +441,10 @@
 	   (string-append-map-ustr-former extract-kana preconv-str)
 	   (if convert-pending-into-kana?
 	       (if residual-kana
-		   (extract-kana residual-kana)
-		   "")
+                 (if (list? (car residual-kana))
+                   (string-append-map extract-kana residual-kana)
+		   (extract-kana residual-kana))
+                 "")
 	       pending)
 	   (string-append-map-ustr-latter extract-kana preconv-str))))))
 
@@ -712,7 +746,9 @@
 		   (res (rk-push-key! rkc key-str)))
 	      (if res
 		  (begin
-		    (ustr-insert-elem! (anthy-context-preconv-ustr ac) res)
+		    (if (list? (car res))
+		      (ustr-insert-seq! (anthy-context-preconv-ustr ac) res)
+		      (ustr-insert-elem! (anthy-context-preconv-ustr ac) res))
 		    (ustr-insert-elem! (anthy-context-raw-ustr ac)
 				       (if (and (intern-key-symbol key-str)
 						(symbol-bound?
@@ -1018,7 +1054,7 @@
 	       (not (shift-key-mask key-state))))
 	     ;; go back to unselected prediction
 	     (anthy-reset-prediction-window ac)
-	     (anthy-check-prediction ac))
+	     (anthy-check-prediction ac #f))
 	    ((and
 	      (ichar-numeric? key)
 	      anthy-select-prediction-by-numeral-key?
@@ -1033,10 +1069,20 @@
 	  (begin
 	    (anthy-context-set-predicting! ac #f)
 	    (anthy-context-set-prediction-index! ac #f)
+	    (if (not anthy-use-prediction?)
+		(anthy-reset-prediction-window ac))
 	    (anthy-proc-input-state ac key key-state)))))))
 
 (define anthy-proc-input-state-with-preedit
   (lambda (ac key key-state)
+    (define (check-auto-conv str)
+      (and
+	str
+	anthy-auto-start-henkan?
+	(string-find japanese-auto-start-henkan-keyword-list str)
+	(begin
+	  (anthy-reset-prediction-window ac)
+	  (anthy-begin-conv ac))))
     (let ((preconv-str (anthy-context-preconv-ustr ac))
 	  (raw-str (anthy-context-raw-ustr ac))
 	  (rkc (anthy-context-rkc ac))
@@ -1048,7 +1094,11 @@
        ((anthy-begin-conv-key? key key-state)
 	(anthy-reset-prediction-window ac)
 	(anthy-begin-conv ac))
-       
+
+       ;; prediction 
+       ((anthy-next-prediction-key? key key-state)
+	(anthy-check-prediction ac #t))
+
        ;; backspace
        ((anthy-backspace-key? key key-state)
 	(if (not (rk-backspace rkc))
@@ -1215,25 +1265,6 @@
 	#f)
 
        (else	
-	;; handle "n1" sequence as "1"
-	(if (and (not (anthy-context-alnum ac))
-		 (not (ichar-alphabetic? key))
-		 (not (string-find
-		       (rk-expect rkc)
-		       (if (= rule anthy-input-rule-kana)
-			   (if (symbol? key)
-			       (symbol->string key)
-			       (charcode->string key))
-			   (if (symbol? key)
-			       (symbol->string key)
-			       (charcode->string (ichar-downcase key)))))))
-	    (let ((pend (rk-pending rkc))
-		  (residual-kana (rk-push-key-last! rkc)))
-	      (if residual-kana
-		  (begin
-		    (ustr-insert-elem! preconv-str residual-kana)
-		    (ustr-insert-elem! raw-str pend)))))
-
 	(if (anthy-context-alnum ac)
 	    (let ((key-str (if (symbol? key)
 			       (if (symbol-bound? key)
@@ -1245,15 +1276,22 @@
 	      (rk-flush rkc) ;; OK to reset rkc here.
 	      (if residual-kana
 		  (begin
-		    (ustr-insert-elem! preconv-str residual-kana)
-		    (ustr-insert-elem! raw-str pend)))
+                    (if (list? (car residual-kana))
+                      (begin
+                        (ustr-insert-seq! preconv-str residual-kana)
+                        (ustr-insert-seq! raw-str (reverse
+                                                    (string-to-list pend))))
+                      (begin
+                        (ustr-insert-elem! preconv-str residual-kana)
+                        (ustr-insert-elem! raw-str pend)))))
 	      (ustr-insert-elem! preconv-str 
 				 (if (= (anthy-context-alnum-type ac)
 					anthy-type-halfwidth-alnum)
 				     (list key-str key-str key-str)
 				     (list (ja-wide key-str) (ja-wide key-str)
 					   (ja-wide key-str))))
-	      (ustr-insert-elem! raw-str key-str))
+	      (ustr-insert-elem! raw-str key-str)
+	      (check-auto-conv key-str))
 	    (let* ((key-str (if (= rule anthy-input-rule-kana)
 	    			(if (symbol? key)
 				    (symbol->string key)
@@ -1272,10 +1310,20 @@
 			(ustr-insert-elem! preconv-str res))
 		    (if (and next-pend
 			     (not (string=? next-pend "")))
-			(ustr-insert-elem! raw-str pend)
+                        (ustr-insert-seq! raw-str
+                                          (reverse (string-to-list pend)))
 			(if (list? (car res))
 			    (begin
-			      (ustr-insert-elem! raw-str pend)
+                              (if (member pend
+                                          (map car
+                                               ja-consonant-syllable-table))
+                                ;; treat consonant having more than one
+                                ;; charactear as one raw-str in this case
+                                (ustr-insert-elem! raw-str pend)
+                                (ustr-insert-seq! raw-str (reverse
+                                                            (string-to-list
+                                                              pend))))
+                              ;; assume key-str as a vowel
 			      (ustr-insert-elem!
 			       raw-str (if (and (intern-key-symbol key-str)
 						(symbol-bound?
@@ -1291,7 +1339,8 @@
 				   (intern-key-symbol key-str)
 				   (symbol-bound? (string->symbol key-str)))
 				  (symbol-value (string->symbol key-str))
-				  key-str))))))))))))))
+				  key-str)))))))
+	      (check-auto-conv (if res (car res) #f)))))))))
 
 (define anthy-context-confirm-kana!
   (lambda (ac)
@@ -1302,7 +1351,9 @@
 	       (residual-kana (rk-peek-terminal-match rkc)))
 	    (if residual-kana
 		(begin
-		  (ustr-insert-elem! preconv-str residual-kana)
+                  (if (list? (car residual-kana))
+                    (ustr-insert-seq! preconv-str residual-kana)
+                    (ustr-insert-elem! preconv-str residual-kana))
 		  (rk-flush rkc)))))))
 
 (define anthy-reset-prediction-window
@@ -1314,15 +1365,25 @@
     (anthy-context-set-prediction-index! ac #f)))
 
 (define anthy-check-prediction
-  (lambda (ac)
+  (lambda (ac force-check?)
     (if (and
 	 (not (anthy-context-converting ac))
 	 (not (anthy-context-transposing ac))
 	 (not (anthy-context-predicting ac)))
-	(let ((preconv-str
-	       (anthy-make-whole-string ac #t (anthy-context-kana-mode ac)))
-	      (ac-id (anthy-context-ac-id ac)))
-	  (if (not (string=? preconv-str ""))
+	(let* ((use-pending-rk-for-prediction? #f)
+	       (preconv-str
+		(anthy-make-whole-string ac #t (anthy-context-kana-mode ac)))
+	       (ac-id (anthy-context-ac-id ac))
+	       (preedit-len
+		(+
+		 (ustr-length (anthy-context-preconv-ustr ac))
+		 (if (not use-pending-rk-for-prediction?)
+		     0
+		     (string-length
+		      (rk-pending (anthy-context-rkc ac)))))))
+	  (if (or
+	       (>= preedit-len anthy-prediction-start-char-count)
+	       force-check?)
 	      (begin
 		(anthy-lib-set-prediction-src-string ac-id preconv-str)
 		(let ((nr (anthy-lib-get-nr-predictions ac-id)))
@@ -1343,8 +1404,9 @@
 	(anthy-proc-input-state-with-preedit ac key key-state)
 	(anthy-proc-input-state-no-preedit ac key key-state))
     (if (and
-         anthy-use-prediction?)
-	 (anthy-check-prediction ac))))
+	 anthy-use-prediction?
+	 (not (anthy-context-predicting ac)))
+	(anthy-check-prediction ac #f))))
 
 (define anthy-separator
   (lambda (ac)
@@ -1388,7 +1450,9 @@
 	   (left-str (ustr-former-seq raw-str)))
       (append left-str
 	      (if residual-kana
-		  (list pending)
+                (if (list? (car residual-kana))
+		  (reverse (string-to-list pending))
+		  (list pending))
 		  '())
 	      right-str))))
 
@@ -1406,7 +1470,9 @@
 	  (if (member (car unconv) preconv)
 	      (let ((start (list-seq-contained? preconv unconv))
 		    (len (length unconv)))
-		(if start
+		(if (and
+                      start
+                      (= (length raw-str) (length preconv))) ;; sanity check
 		    (anthy-make-raw-string
 		     (reverse (sublist-rel raw-str start len))
 		     (if (or
