@@ -1,6 +1,6 @@
 ;;; byeoru-custom.scm: Customization variables for byeoru.scm
 ;;;
-;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2010 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -96,8 +96,8 @@
   (cons 'choice
 	(map (lambda (entry)
 	       (let ((id (car entry))
-		     (label (nth 2 entry))
-		     (short-desc (nth 3 entry)))
+		     (label (list-ref entry 2))
+		     (short-desc (list-ref entry 3)))
 		 (list id
 		       label
 		       short-desc)))
